@@ -51,7 +51,6 @@
 #define PATH_PASSWD "/etc/passwd"
 #define PATH_SHADOW "/etc/shadow"
 #define PATH_GROUP "/etc/group"
-#define PATH_GDM_CUSTOM "/etc/gdm/custom.conf"
 
 enum {
         PROP_0,
@@ -62,6 +61,7 @@ struct DaemonPrivate {
         GDBusConnection *bus_connection;
 
         GHashTable *users;
+        gsize number_of_normal_users;
         GList *explicitly_requested_users;
 
         User *autologin;
@@ -87,8 +87,10 @@ G_DEFINE_TYPE_WITH_CODE (Daemon, daemon, ACCOUNTS_TYPE_ACCOUNTS_SKELETON, G_IMPL
 
 #define DAEMON_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_DAEMON, DaemonPrivate))
 
+G_DEFINE_AUTOPTR_CLEANUP_FUNC (Daemon, g_object_unref)
+
 static const GDBusErrorEntry accounts_error_entries[] =
-{ 
+{
         { ERROR_FAILED, "org.freedesktop.Accounts.Error.Failed" },
         { ERROR_USER_EXISTS, "org.freedesktop.Accounts.Error.UserExists" },
         { ERROR_USER_DOES_NOT_EXIST, "org.freedesktop.Accounts.Error.UserDoesNotExist" },
@@ -140,6 +142,19 @@ error_get_type (void)
 #define MAX_LOCAL_USERS 50
 #endif
 
+static void
+remove_cache_files (const gchar *user_name)
+{
+        g_autofree gchar *user_filename = NULL;
+        g_autofree gchar *icon_filename = NULL;
+
+        user_filename = g_build_filename (USERDIR, user_name, NULL);
+        g_remove (user_filename);
+
+        icon_filename = g_build_filename (ICONDIR, user_name, NULL);
+        g_remove (icon_filename);
+}
+
 static struct passwd *
 entry_generator_fgetpwent (Daemon       *daemon,
                            GHashTable   *users,
@@ -241,12 +256,10 @@ entry_generator_cachedir (Daemon       *daemon,
                           struct spwd **shadow_entry)
 {
         struct passwd *pwent;
-        const gchar *name;
-        GError *error = NULL;
-        gchar *filename;
+        g_autoptr(GError) error = NULL;
         gboolean regular;
         GHashTableIter iter;
-        GKeyFile *key_file;
+        const gchar *name;
         User *user;
         GDir *dir;
 
@@ -256,7 +269,6 @@ entry_generator_cachedir (Daemon       *daemon,
                 if (error != NULL) {
                         if (!g_error_matches (error, G_FILE_ERROR, G_FILE_ERROR_NOENT))
                                 g_warning ("couldn't list user cache directory: %s", USERDIR);
-                        g_error_free (error);
                         return NULL;
                 }
         }
@@ -269,6 +281,9 @@ entry_generator_cachedir (Daemon       *daemon,
          */
         dir = *state;
         while (TRUE) {
+                const gchar *name;
+                g_autofree gchar *filename = NULL;
+
                 name = g_dir_read_name (dir);
                 if (name == NULL)
                         break;
@@ -276,16 +291,21 @@ entry_generator_cachedir (Daemon       *daemon,
                 /* Only load files in this directory */
                 filename = g_build_filename (USERDIR, name, NULL);
                 regular = g_file_test (filename, G_FILE_TEST_IS_REGULAR);
-                g_free (filename);
 
                 if (regular) {
+                        errno = 0;
                         pwent = getpwnam (name);
-                        if (pwent == NULL) {
-                                g_debug ("user '%s' in cache dir but not present on system", name);
-                        } else {
+                        if (pwent != NULL) {
                                 *shadow_entry = getspnam (pwent->pw_name);
 
                                 return pwent;
+                        } else if (errno == 0) {
+                                g_debug ("user '%s' in cache dir but not present on system, removing", name);
+                                remove_cache_files (name);
+                        }
+                        else {
+                                g_warning ("failed to check if user '%s' in cache dir is present on system: %s",
+                                  name, g_strerror (errno));
                         }
                 }
         }
@@ -296,12 +316,13 @@ entry_generator_cachedir (Daemon       *daemon,
         /* Update all the users from the files in the cache dir */
         g_hash_table_iter_init (&iter, users);
         while (g_hash_table_iter_next (&iter, (gpointer *)&name, (gpointer *)&user)) {
+                g_autofree gchar *filename = NULL;
+                g_autoptr(GKeyFile) key_file = NULL;
+
                 filename = g_build_filename (USERDIR, name, NULL);
                 key_file = g_key_file_new ();
                 if (g_key_file_load_from_file (key_file, filename, 0, NULL))
                         user_update_from_keyfile (user, key_file);
-                g_key_file_unref (key_file);
-                g_free (filename);
         }
 
         *state = NULL;
@@ -356,7 +377,7 @@ entry_generator_requested_users (Daemon       *daemon,
 static void
 load_entries (Daemon             *daemon,
               GHashTable         *users,
-              gboolean            allow_system_users,
+              gboolean            explicitly_requested,
               EntryGeneratorFunc  entry_generator)
 {
         gpointer generator_state = NULL;
@@ -373,29 +394,37 @@ load_entries (Daemon             *daemon,
                         break;
 
                 /* Skip system users... */
-                if (!allow_system_users && !user_classify_is_human (pwent->pw_uid, pwent->pw_name, pwent->pw_shell, spent? spent->sp_pwdp : NULL)) {
+                if (!explicitly_requested && !user_classify_is_human (pwent->pw_uid, pwent->pw_name, pwent->pw_shell, spent? spent->sp_pwdp : NULL)) {
                         g_debug ("skipping user: %s", pwent->pw_name);
                         continue;
                 }
 
-                /* ignore duplicate entries */
-                if (g_hash_table_lookup (users, pwent->pw_name)) {
-                        continue;
-                }
+                /* Only process users that haven't been processed yet.
+                 * We do always make sure entries get promoted
+                 * to "cached" status if they are supposed to be
+                 */
+
+                user = g_hash_table_lookup (users, pwent->pw_name);
 
-                user = g_hash_table_lookup (daemon->priv->users, pwent->pw_name);
                 if (user == NULL) {
-                        user = user_new (daemon, pwent->pw_uid);
-                } else {
-                        g_object_ref (user);
+                        user = g_hash_table_lookup (daemon->priv->users, pwent->pw_name);
+                        if (user == NULL) {
+                                user = user_new (daemon, pwent->pw_uid);
+                        } else {
+                                g_object_ref (user);
+                        }
+
+                        /* freeze & update users not already in the new list */
+                        g_object_freeze_notify (G_OBJECT (user));
+                        user_update_from_pwent (user, pwent, spent);
+
+                        g_hash_table_insert (users, g_strdup (user_get_user_name (user)), user);
+                        g_debug ("loaded user: %s", user_get_user_name (user));
                 }
 
-                /* freeze & update users not already in the new list */
-                g_object_freeze_notify (G_OBJECT (user));
-                user_update_from_pwent (user, pwent, spent);
-
-                g_hash_table_insert (users, g_strdup (user_get_user_name (user)), user);
-                g_debug ("loaded user: %s", user_get_user_name (user));
+                if (!explicitly_requested) {
+                        user_set_cached (user, TRUE);
+                }
         }
 
         /* Generator should have cleaned up */
@@ -414,10 +443,13 @@ create_users_hash_table (void)
 static void
 reload_users (Daemon *daemon)
 {
+        AccountsAccounts *accounts = ACCOUNTS_ACCOUNTS (daemon);
+        gboolean had_no_users, has_no_users, had_multiple_users, has_multiple_users;
         GHashTable *users;
         GHashTable *old_users;
         GHashTable *local;
         GHashTableIter iter;
+        gsize number_of_normal_users = 0;
         gpointer name;
         User *user;
 
@@ -445,13 +477,27 @@ reload_users (Daemon *daemon)
 
         wtmp_helper_update_login_frequencies (users);
 
-        /* Mark which users are local, which are not */
+        /* Count the non-system users. Mark which users are local, which are not. */
         g_hash_table_iter_init (&iter, users);
-        while (g_hash_table_iter_next (&iter, &name, (gpointer *)&user))
+        while (g_hash_table_iter_next (&iter, &name, (gpointer *)&user)) {
+                if (!user_get_system_account (user))
+                        number_of_normal_users++;
                 user_update_local_account_property (user, g_hash_table_lookup (local, name) != NULL);
-
+        }
         g_hash_table_destroy (local);
 
+        had_no_users = accounts_accounts_get_has_no_users (accounts);
+        has_no_users = number_of_normal_users == 0;
+
+        if (had_no_users != has_no_users)
+                accounts_accounts_set_has_no_users (accounts, has_no_users);
+
+        had_multiple_users = accounts_accounts_get_has_multiple_users (accounts);
+        has_multiple_users = number_of_normal_users > 1;
+
+        if (had_multiple_users != has_multiple_users)
+                accounts_accounts_set_has_multiple_users (accounts, has_multiple_users);
+
         /* Swap out the users */
         old_users = daemon->priv->users;
         daemon->priv->users = users;
@@ -459,7 +505,11 @@ reload_users (Daemon *daemon)
         /* Remove all the old users */
         g_hash_table_iter_init (&iter, old_users);
         while (g_hash_table_iter_next (&iter, &name, (gpointer *)&user)) {
-                if (!g_hash_table_lookup (users, name)) {
+                User *refreshed_user;
+
+                refreshed_user = g_hash_table_lookup (users, name);
+
+                if (!refreshed_user || !user_get_cached (refreshed_user)) {
                         user_unregister (user);
                         accounts_accounts_emit_user_deleted (ACCOUNTS_ACCOUNTS (daemon),
                                                              user_get_object_path (user));
@@ -469,7 +519,11 @@ reload_users (Daemon *daemon)
         /* Register all the new users */
         g_hash_table_iter_init (&iter, users);
         while (g_hash_table_iter_next (&iter, &name, (gpointer *)&user)) {
-                if (!g_hash_table_lookup (old_users, name)) {
+                User *stale_user;
+
+                stale_user = g_hash_table_lookup (old_users, name);
+
+                if (!stale_user || !user_get_cached (stale_user) && user_get_cached (user)) {
                         user_register (user);
                         accounts_accounts_emit_user_added (ACCOUNTS_ACCOUNTS (daemon),
                                                            user_get_object_path (user));
@@ -497,18 +551,16 @@ static gboolean load_autologin (Daemon    *daemon,
 static gboolean
 reload_autologin_timeout (Daemon *daemon)
 {
+        AccountsAccounts *accounts = ACCOUNTS_ACCOUNTS (daemon);
         gboolean enabled;
-        gchar *name = NULL;
-        GError *error = NULL;
+        g_autofree gchar *name = NULL;
+        g_autoptr(GError) error = NULL;
         User *user = NULL;
 
         daemon->priv->autologin_id = 0;
 
         if (!load_autologin (daemon, &name, &enabled, &error)) {
                 g_debug ("failed to load gdms custom.conf: %s", error->message);
-                g_error_free (error);
-                g_free (name);
-
                 return FALSE;
         }
 
@@ -518,12 +570,16 @@ reload_autologin_timeout (Daemon *daemon)
         if (daemon->priv->autologin != NULL && daemon->priv->autologin != user) {
                 g_object_set (daemon->priv->autologin, "automatic-login", FALSE, NULL);
                 g_signal_emit_by_name (daemon->priv->autologin, "changed", 0);
-                g_object_unref (daemon->priv->autologin);
-                daemon->priv->autologin = NULL;
+                g_clear_object (&daemon->priv->autologin);
         }
 
         if (enabled) {
+                const gchar *users[2];
+
                 g_debug ("automatic login is enabled for '%s'", name);
+                users[0] = user_get_object_path (user);
+                users[1] = NULL;
+                accounts_accounts_set_automatic_login_users (accounts, users);
                 if (daemon->priv->autologin != user) {
                         g_object_set (user, "automatic-login", TRUE, NULL);
                         daemon->priv->autologin = g_object_ref (user);
@@ -532,10 +588,9 @@ reload_autologin_timeout (Daemon *daemon)
         }
         else {
                 g_debug ("automatic login is disabled");
+                accounts_accounts_set_automatic_login_users (accounts, NULL);
         }
 
-        g_free (name);
-
         return FALSE;
 }
 
@@ -613,9 +668,9 @@ setup_monitor (Daemon             *daemon,
                const gchar        *path,
                FileChangeCallback *callback)
 {
-        GError *error = NULL;
-        GFile *file;
+        g_autoptr(GFile) file = NULL;
         GFileMonitor *monitor;
+        g_autoptr(GError) error = NULL;
 
         if (!path) {
                 return NULL;
@@ -626,16 +681,15 @@ setup_monitor (Daemon             *daemon,
                                        G_FILE_MONITOR_NONE,
                                        NULL,
                                        &error);
-        if (monitor != NULL) {
-                g_signal_connect (monitor,
-                                  "changed",
-                                  G_CALLBACK (callback),
-                                  daemon);
-        } else {
+        if (monitor == NULL) {
                 g_warning ("Unable to monitor %s: %s", path, error->message);
-                g_error_free (error);
+                return NULL;
         }
-        g_object_unref (file);
+
+        g_signal_connect (monitor,
+                          "changed",
+                          G_CALLBACK (callback),
+                          daemon);
 
         return monitor;
 }
@@ -694,60 +748,46 @@ daemon_finalize (GObject *object)
 static gboolean
 register_accounts_daemon (Daemon *daemon)
 {
-        GError *error = NULL;
+        g_autoptr(GError) error = NULL;
 
         daemon->priv->authority = polkit_authority_get_sync (NULL, &error);
-
         if (daemon->priv->authority == NULL) {
-                if (error != NULL) {
+                if (error != NULL)
                         g_critical ("error getting polkit authority: %s", error->message);
-                        g_error_free (error);
-                }
-                goto error;
+                return FALSE;
         }
 
         daemon->priv->bus_connection = g_bus_get_sync (G_BUS_TYPE_SYSTEM, NULL, &error);
         if (daemon->priv->bus_connection == NULL) {
-                if (error != NULL) {
+                if (error != NULL)
                         g_critical ("error getting system bus: %s", error->message);
-                        g_error_free (error);
-                }
-                goto error;
+                return FALSE;
         }
 
         if (!g_dbus_interface_skeleton_export (G_DBUS_INTERFACE_SKELETON (daemon),
                                                daemon->priv->bus_connection,
                                                "/org/freedesktop/Accounts",
                                                &error)) {
-                if (error != NULL) {
+                if (error != NULL)
                         g_critical ("error exporting interface: %s", error->message);
-                        g_error_free (error);
-                }
-                goto error;     
+                return FALSE;
         }
 
         return TRUE;
-
- error:
-        return FALSE;
 }
 
 Daemon *
 daemon_new (void)
 {
-        Daemon *daemon;
+        g_autoptr(Daemon) daemon = NULL;
 
         daemon = DAEMON (g_object_new (TYPE_DAEMON, NULL));
 
         if (!register_accounts_daemon (DAEMON (daemon))) {
-                g_object_unref (daemon);
-                goto error;
+                return NULL;
         }
 
-        return daemon;
-
- error:
-        return NULL;
+        return g_steal_pointer (&daemon);
 }
 
 static void
@@ -757,15 +797,13 @@ throw_error (GDBusMethodInvocation *context,
              ...)
 {
         va_list args;
-        gchar *message;
+        g_autofree gchar *message = NULL;
 
         va_start (args, format);
         message = g_strdup_vprintf (format, args);
         va_end (args);
 
         g_dbus_method_invocation_return_error (context, ERROR, error_code, "%s", message);
-
-        g_free (message);
 }
 
 static User *
@@ -919,7 +957,7 @@ static gboolean
 finish_list_cached_users (gpointer user_data)
 {
         ListUserData *data = user_data;
-        GPtrArray *object_paths;
+        g_autoptr(GPtrArray) object_paths = NULL;
         GHashTableIter iter;
         const gchar *name;
         User *user;
@@ -938,6 +976,11 @@ finish_list_cached_users (gpointer user_data)
                         continue;
                 }
 
+                if (!user_get_cached (user)) {
+                        g_debug ("user %s %ld not cached", name, (long) uid);
+                        continue;
+                }
+
                 g_debug ("user %s %ld not excluded", name, (long) uid);
                 g_ptr_array_add (object_paths, (gpointer) user_get_object_path (user));
         }
@@ -945,8 +988,6 @@ finish_list_cached_users (gpointer user_data)
 
         accounts_accounts_complete_list_cached_users (NULL, data->context, (const gchar * const *) object_paths->pdata);
 
-        g_ptr_array_free (object_paths, TRUE);
-
         list_user_data_free (data);
 
         return FALSE;
@@ -982,8 +1023,8 @@ static void
 cache_user (Daemon *daemon,
             User   *user)
 {
-        gchar       *filename;
-        const char  *user_name;
+        g_autofree gchar *filename = NULL;
+        const gchar *user_name;
 
         /* Always use the canonical user name looked up */
         user_name = user_get_user_name (user);
@@ -992,8 +1033,6 @@ cache_user (Daemon *daemon,
         if (!g_file_test (filename, G_FILE_TEST_EXISTS)) {
                 user_save (user);
         }
-
-        g_free (filename);
 }
 
 typedef struct {
@@ -1021,12 +1060,11 @@ daemon_create_user_authorized_cb (Daemon                *daemon,
 {
         CreateUserData *cd = data;
         User *user;
-        GError *error;
+        g_autoptr(GError) error = NULL;
         const gchar *argv[9];
 
         if (getpwnam (cd->user_name) != NULL) {
                 throw_error (context, ERROR_USER_EXISTS, "A user with name '%s' already exists", cd->user_name);
-
                 return;
         }
 
@@ -1037,8 +1075,15 @@ daemon_create_user_authorized_cb (Daemon                *daemon,
         argv[2] = "-c";
         argv[3] = cd->real_name;
         if (cd->account_type == ACCOUNT_TYPE_ADMINISTRATOR) {
+                g_autofree gchar *admin_groups = NULL;
+
+                admin_groups = g_strdup (ADMIN_GROUP);
+                if (EXTRA_ADMIN_GROUPS != NULL && EXTRA_ADMIN_GROUPS[0] != '\0')
+                        admin_groups = g_strconcat (admin_groups, ",",
+                                                    EXTRA_ADMIN_GROUPS, NULL);
+
                 argv[4] = "-G";
-                argv[5] = ADMIN_GROUP;
+                argv[5] = admin_groups;
                 argv[6] = "--";
                 argv[7] = cd->user_name;
                 argv[8] = NULL;
@@ -1053,10 +1098,8 @@ daemon_create_user_authorized_cb (Daemon                *daemon,
                 return;
         }
 
-        error = NULL;
         if (!spawn_with_login_uid (context, argv, &error)) {
                 throw_error (context, ERROR_FAILED, "running '%s' failed: %s", argv[0], error->message);
-                g_error_free (error);
                 return;
         }
 
@@ -1155,7 +1198,6 @@ daemon_uncache_user_authorized_cb (Daemon                *daemon,
                                    gpointer               data)
 {
         const gchar *user_name = data;
-        gchar       *filename;
         User        *user;
 
         sys_log (context, "uncache user '%s'", user_name);
@@ -1170,13 +1212,9 @@ daemon_uncache_user_authorized_cb (Daemon                *daemon,
         /* Always use the canonical user name looked up */
         user_name = user_get_user_name (user);
 
-        filename = g_build_filename (USERDIR, user_name, NULL);
-        g_remove (filename);
-        g_free (filename);
+        remove_cache_files (user_name);
 
-        filename = g_build_filename (ICONDIR, user_name, NULL);
-        g_remove (filename);
-        g_free (filename);
+        user_set_cached (user, FALSE);
 
         accounts_accounts_complete_uncache_user (NULL, context);
 
@@ -1215,41 +1253,31 @@ daemon_delete_user_authorized_cb (Daemon                *daemon,
 
 {
         DeleteUserData *ud = data;
-        GError *error;
-        gchar *filename;
+        g_autoptr(GError) error = NULL;
         struct passwd *pwent;
         const gchar *argv[6];
+        User *user;
 
         pwent = getpwuid (ud->uid);
 
         if (pwent == NULL) {
                 throw_error (context, ERROR_USER_DOES_NOT_EXIST, "No user with uid %d found", ud->uid);
-
                 return;
         }
 
         sys_log (context, "delete user '%s' (%d)", pwent->pw_name, ud->uid);
 
-        if (daemon->priv->autologin != NULL) {
-                User *user;
+        user = daemon_local_find_user_by_id (daemon, ud->uid);
 
-                user = daemon_local_find_user_by_id (daemon, ud->uid);
-
-                g_assert (user != NULL);
+        if (user != NULL) {
+                user_set_cached (user, FALSE);
 
                 if (daemon->priv->autologin == user) {
                         daemon_local_set_automatic_login (daemon, user, FALSE, NULL);
                 }
-
         }
 
-        filename = g_build_filename (USERDIR, pwent->pw_name, NULL);
-        g_remove (filename);
-        g_free (filename);
-
-        filename = g_build_filename (ICONDIR, pwent->pw_name, NULL);
-        g_remove (filename);
-        g_free (filename);
+        remove_cache_files (pwent->pw_name);
 
         argv[0] = "/usr/sbin/userdel";
         if (ud->remove_files) {
@@ -1266,10 +1294,8 @@ daemon_delete_user_authorized_cb (Daemon                *daemon,
                 argv[4] = NULL;
         }
 
-        error = NULL;
         if (!spawn_with_login_uid (context, argv, &error)) {
                 throw_error (context, ERROR_FAILED, "running '%s' failed: %s", argv[0], error->message);
-                g_error_free (error);
                 return;
         }
 
@@ -1337,16 +1363,12 @@ check_auth_cb (PolkitAuthority *authority,
 {
         CheckAuthData *cad = data;
         PolkitAuthorizationResult *result;
-        GError *error;
-        gboolean is_authorized;
+        g_autoptr(GError) error = NULL;
+        gboolean is_authorized = FALSE;
 
-        is_authorized = FALSE;
-
-        error = NULL;
         result = polkit_authority_check_authorization_finish (authority, res, &error);
         if (error) {
                 throw_error (cad->context, ERROR_PERMISSION_DENIED, "Not authorized: %s", error->message);
-                g_error_free (error);
         }
         else {
                 if (polkit_authorization_result_get_is_authorized (result)) {
@@ -1418,25 +1440,21 @@ load_autologin (Daemon      *daemon,
                 gboolean    *enabled,
                 GError     **error)
 {
-        GKeyFile *keyfile;
-        GError *local_error;
-        gchar *string;
+        g_autoptr(GKeyFile) keyfile = NULL;
+        GError *local_error = NULL;
+        g_autofree gchar *string = NULL;
 
         keyfile = g_key_file_new ();
         if (!g_key_file_load_from_file (keyfile,
                                         PATH_GDM_CUSTOM,
                                         G_KEY_FILE_KEEP_COMMENTS,
                                         error)) {
-                g_key_file_free (keyfile);
                 return FALSE;
         }
 
-        local_error = NULL;
         string = g_key_file_get_string (keyfile, "daemon", "AutomaticLoginEnable", &local_error);
         if (local_error) {
                 g_propagate_error (error, local_error);
-                g_key_file_free (keyfile);
-                g_free (string);
                 return FALSE;
         }
         if (string != NULL && (g_ascii_strcasecmp (string, "true") == 0 || strcmp (string, "1") == 0)) {
@@ -1445,17 +1463,13 @@ load_autologin (Daemon      *daemon,
         else {
                 *enabled = FALSE;
         }
-        g_free (string);
 
         *name = g_key_file_get_string (keyfile, "daemon", "AutomaticLogin", &local_error);
         if (local_error) {
                 g_propagate_error (error, local_error);
-                g_key_file_free (keyfile);
                 return FALSE;
         }
 
-        g_key_file_free (keyfile);
-
         return TRUE;
 }
 
@@ -1465,8 +1479,8 @@ save_autologin (Daemon      *daemon,
                 gboolean     enabled,
                 GError     **error)
 {
-        GKeyFile *keyfile;
-        gchar *data;
+        g_autoptr(GKeyFile) keyfile = NULL;
+        g_autofree gchar *data = NULL;
         gboolean result;
 
         keyfile = g_key_file_new ();
@@ -1474,7 +1488,6 @@ save_autologin (Daemon      *daemon,
                                         PATH_GDM_CUSTOM,
                                         G_KEY_FILE_KEEP_COMMENTS,
                                         error)) {
-                g_key_file_free (keyfile);
                 return FALSE;
         }
 
@@ -1484,9 +1497,6 @@ save_autologin (Daemon      *daemon,
         data = g_key_file_to_data (keyfile, NULL, NULL);
         result = g_file_set_contents (PATH_GDM_CUSTOM, data, -1, error);
 
-        g_key_file_free (keyfile);
-        g_free (data);
-
         return result;
 }
 
@@ -1511,8 +1521,7 @@ daemon_local_set_automatic_login (Daemon    *daemon,
         if (daemon->priv->autologin != NULL) {
                 g_object_set (daemon->priv->autologin, "automatic-login", FALSE, NULL);
                 g_signal_emit_by_name (daemon->priv->autologin, "changed", 0);
-                g_object_unref (daemon->priv->autologin);
-                daemon->priv->autologin = NULL;
+                g_clear_object (&daemon->priv->autologin);
         }
 
         if (enabled) {
@@ -1528,7 +1537,7 @@ daemon_local_set_automatic_login (Daemon    *daemon,
 GHashTable *
 daemon_get_extension_ifaces (Daemon *daemon)
 {
-  return daemon->priv->extension_ifaces;
+        return daemon->priv->extension_ifaces;
 }
 
 static void
@@ -1537,7 +1546,7 @@ get_property (GObject    *object,
               GValue     *value,
               GParamSpec *pspec)
 {
-       switch (prop_id) {
+        switch (prop_id) {
         case PROP_DAEMON_VERSION:
                 g_value_set_string (value, VERSION);
                 break;
@@ -1554,7 +1563,7 @@ set_property (GObject      *object,
               const GValue *value,
               GParamSpec   *pspec)
 {
-       switch (prop_id) {
+        switch (prop_id) {
         case PROP_DAEMON_VERSION:
                 g_assert_not_reached ();
                 break;
