@@ -106,10 +106,12 @@ struct User {
         GVariant     *login_history;
         gchar        *icon_file;
         gchar        *default_icon_file;
+        gboolean      account_expiration_policy_known;
         gboolean      locked;
         gboolean      automatic_login;
         gboolean      system_account;
         gboolean      local_account;
+        gboolean      cached;
 
         guint        *extension_ids;
         guint         n_extension_ids;
@@ -155,7 +157,7 @@ user_update_from_pwent (User          *user,
                         struct passwd *pwent,
                         struct spwd   *spent)
 {
-        gchar *real_name;
+        g_autofree gchar *real_name = NULL;
         gboolean changed;
         const gchar *passwd;
         gboolean locked;
@@ -192,8 +194,7 @@ user_update_from_pwent (User          *user,
                 }
 
                 if (real_name && real_name[0] == '\0') {
-                        g_free (real_name);
-                        real_name = NULL;
+                        g_clear_pointer (&real_name, g_free);
                 }
         }
         else {
@@ -201,13 +202,10 @@ user_update_from_pwent (User          *user,
         }
         if (g_strcmp0 (real_name, user->real_name) != 0) {
                 g_free (user->real_name);
-                user->real_name = real_name;
+                user->real_name = g_steal_pointer (&real_name);
                 changed = TRUE;
                 g_object_notify (G_OBJECT (user), "real-name");
         }
-        else {
-                g_free (real_name);
-        }
 
         /* UID */
         if (pwent->pw_uid != user->uid) {
@@ -287,6 +285,7 @@ user_update_from_pwent (User          *user,
                 user->max_days_between_changes = spent->sp_max;
                 user->days_to_warn  = spent->sp_warn;
                 user->days_after_expiration_until_lock = spent->sp_inact;
+                user->account_expiration_policy_known = TRUE;
         }
 
         if (user->password_mode != mode) {
@@ -366,6 +365,7 @@ user_update_from_keyfile (User     *user,
 
         g_clear_pointer (&user->keyfile, g_key_file_unref);
         user->keyfile = g_key_file_ref (keyfile);
+        user_set_cached (user, TRUE);
 
         g_object_thaw_notify (G_OBJECT (user));
 }
@@ -415,40 +415,38 @@ user_save_to_keyfile (User     *user,
                 g_key_file_set_string (keyfile, "User", "Icon", user->icon_file);
 
         g_key_file_set_boolean (keyfile, "User", "SystemAccount", user->system_account);
+
+        user_set_cached (user, TRUE);
 }
 
 static void
 save_extra_data (User *user)
 {
-        gchar *filename;
-        gchar *data;
-        GError *error;
+        g_autofree gchar *data = NULL;
+        g_autofree gchar *filename = NULL;
+        g_autoptr(GError) error = NULL;
 
         user_save_to_keyfile (user, user->keyfile);
 
-        error = NULL;
         data = g_key_file_to_data (user->keyfile, NULL, &error);
-        if (error == NULL) {
-                filename = g_build_filename (USERDIR,
-                                             user->user_name,
-                                             NULL);
-                g_file_set_contents (filename, data, -1, &error);
-                g_free (filename);
-                g_free (data);
-        }
-        if (error) {
+        if (data == NULL) {
                 g_warning ("Saving data for user %s failed: %s",
                            user->user_name, error->message);
-                g_error_free (error);
+                return;
         }
+
+        filename = g_build_filename (USERDIR,
+                                     user->user_name,
+                                     NULL);
+        g_file_set_contents (filename, data, -1, &error);
 }
 
 static void
 move_extra_data (const gchar *old_name,
                  const gchar *new_name)
 {
-        gchar *old_filename;
-        gchar *new_filename;
+        g_autofree gchar *old_filename = NULL;
+        g_autofree gchar *new_filename = NULL;
 
         old_filename = g_build_filename (USERDIR,
                                          old_name, NULL);
@@ -456,9 +454,6 @@ move_extra_data (const gchar *old_name,
                                          new_name, NULL);
 
         g_rename (old_filename, new_filename);
-
-        g_free (old_filename);
-        g_free (new_filename);
 }
 
 static GVariant *
@@ -468,15 +463,13 @@ user_extension_get_value (User                    *user,
 {
         const GVariantType *type = G_VARIANT_TYPE (property->signature);
         GVariant *value;
-        gchar *printed;
+        g_autofree gchar *printed = NULL;
         gint i;
 
         /* First, try to get the value from the keyfile */
         printed = g_key_file_get_value (user->keyfile, interface->name, property->name, NULL);
         if (printed) {
                 value = g_variant_parse (type, printed, NULL, NULL, NULL);
-                g_free (printed);
-
                 if (value != NULL)
                         return value;
         }
@@ -507,13 +500,12 @@ user_extension_get_property (User                  *user,
                              GDBusMethodInvocation *invocation)
 {
         const GDBusPropertyInfo *property = g_dbus_method_invocation_get_property_info (invocation);
-        GVariant *value;
+        g_autoptr(GVariant) value = NULL;
 
         value = user_extension_get_value (user, interface, property);
 
         if (value) {
                 g_dbus_method_invocation_return_value (invocation, g_variant_new ("(v)", value));
-                g_variant_unref (value);
         }
         else {
                 g_dbus_method_invocation_return_error (invocation, G_DBUS_ERROR, G_DBUS_ERROR_INVALID_ARGS,
@@ -534,14 +526,12 @@ user_extension_get_all_properties (User                  *user,
         g_variant_builder_init (&builder, G_VARIANT_TYPE_VARDICT);
         for (i = 0; interface->properties && interface->properties[i]; i++) {
                 GDBusPropertyInfo *property = interface->properties[i];
-                GVariant *value;
+                g_autoptr(GVariant) value = NULL;
 
                 value = user_extension_get_value (user, interface, property);
 
-                if (value) {
+                if (value)
                         g_variant_builder_add (&builder, "{sv}", property->name, value);
-                        g_variant_unref (value);
-                }
         }
 
         g_dbus_method_invocation_return_value (invocation, g_variant_new ("(a{sv})", &builder));
@@ -554,9 +544,9 @@ user_extension_set_property (User                  *user,
                              GDBusMethodInvocation *invocation)
 {
         const GDBusPropertyInfo *property = g_dbus_method_invocation_get_property_info (invocation);
-        GVariant *value;
-        gchar *printed;
-        gchar *prev;
+        g_autoptr(GVariant) value = NULL;
+        g_autofree gchar *printed = NULL;
+        g_autofree gchar *prev = NULL;
 
         g_variant_get_child (g_dbus_method_invocation_get_parameters (invocation), 2, "v", &value);
 
@@ -586,10 +576,6 @@ user_extension_set_property (User                  *user,
                 save_extra_data (user);
         }
 
-        g_variant_unref (value);
-        g_free (printed);
-        g_free (prev);
-
         g_dbus_method_invocation_return_value (invocation, g_variant_new ("()"));
 }
 
@@ -728,14 +714,12 @@ compute_object_path (User *user)
 void
 user_register (User *user)
 {
-        GError *error = NULL;
+        g_autoptr(GError) error = NULL;
 
         user->system_bus_connection = g_bus_get_sync (G_BUS_TYPE_SYSTEM, NULL, &error);
         if (user->system_bus_connection == NULL) {
-                if (error != NULL) {
+                if (error != NULL)
                         g_critical ("error getting system bus: %s", error->message);
-                        g_error_free (error);
-                }
                 return;
         }
 
@@ -745,10 +729,8 @@ user_register (User *user)
                                                user->system_bus_connection,
                                                user->object_path,
                                                &error)) {
-                if (error != NULL) {
+                if (error != NULL)
                         g_critical ("error exporting user object: %s", error->message);
-                        g_error_free (error);
-                }
                 return;
         }
 
@@ -837,6 +819,19 @@ user_get_shell(User *user)
 	return user->shell;
 }
 
+gboolean
+user_get_cached (User *user)
+{
+        return user->cached;
+}
+
+void
+user_set_cached (User     *user,
+                 gboolean  cached)
+{
+        user->cached = cached;
+}
+
 static void
 throw_error (GDBusMethodInvocation *context,
              gint                   error_code,
@@ -844,15 +839,13 @@ throw_error (GDBusMethodInvocation *context,
              ...)
 {
         va_list args;
-        gchar *message;
+        g_autofree gchar *message = NULL;
 
         va_start (args, format);
         message = g_strdup_vprintf (format, args);
         va_end (args);
 
         g_dbus_method_invocation_return_error (context, ERROR, error_code, "%s", message);
-
-        g_free (message);
 }
 
 static void
@@ -863,7 +856,7 @@ user_change_real_name_authorized_cb (Daemon                *daemon,
 
 {
         gchar *name = data;
-        GError *error;
+        g_autoptr(GError) error = NULL;
         const gchar *argv[6];
 
         if (g_strcmp0 (user->real_name, name) != 0) {
@@ -878,10 +871,8 @@ user_change_real_name_authorized_cb (Daemon                *daemon,
                 argv[4] = user->user_name;
                 argv[5] = NULL;
 
-                error = NULL;
                 if (!spawn_with_login_uid (context, argv, &error)) {
                         throw_error (context, ERROR_FAILED, "running '%s' failed: %s", argv[0], error->message);
-                        g_error_free (error);
                         return;
                 }
 
@@ -936,7 +927,7 @@ user_change_user_name_authorized_cb (Daemon                *daemon,
 {
         gchar *name = data;
         gchar *old_name;
-        GError *error;
+        g_autoptr(GError) error = NULL;
         const gchar *argv[6];
 
         if (g_strcmp0 (user->user_name, name) != 0) {
@@ -952,10 +943,8 @@ user_change_user_name_authorized_cb (Daemon                *daemon,
                 argv[4] = user->user_name;
                 argv[5] = NULL;
 
-                error = NULL;
                 if (!spawn_with_login_uid (context, argv, &error)) {
                         throw_error (context, ERROR_FAILED, "running '%s' failed: %s", argv[0], error->message);
-                        g_error_free (error);
                         return;
                 }
 
@@ -1164,6 +1153,11 @@ user_get_password_expiration_policy_authorized_cb (Daemon                *daemon
                                                    gpointer               data)
 
 {
+        if (!user->account_expiration_policy_known) {
+                throw_error (context, ERROR_NOT_SUPPORTED, "account expiration policy unknown to accounts service");
+                return;
+        }
+
         accounts_user_complete_get_password_expiration_policy (ACCOUNTS_USER (user),
                                                                context,
                                                                user->expiration_time,
@@ -1267,7 +1261,7 @@ user_change_home_dir_authorized_cb (Daemon                *daemon,
 
 {
         gchar *home_dir = data;
-        GError *error;
+        g_autoptr(GError) error = NULL;
         const gchar *argv[7];
 
         if (g_strcmp0 (user->home_dir, home_dir) != 0) {
@@ -1283,10 +1277,8 @@ user_change_home_dir_authorized_cb (Daemon                *daemon,
                 argv[5] = user->user_name;
                 argv[6] = NULL;
 
-                error = NULL;
                 if (!spawn_with_login_uid (context, argv, &error)) {
                         throw_error (context, ERROR_FAILED, "running '%s' failed: %s", argv[0], error->message);
-                        g_error_free (error);
                         return;
                 }
 
@@ -1329,7 +1321,7 @@ user_change_shell_authorized_cb (Daemon                *daemon,
 
 {
         gchar *shell = data;
-        GError *error;
+        g_autoptr(GError) error = NULL;
         const gchar *argv[6];
 
         if (g_strcmp0 (user->shell, shell) != 0) {
@@ -1344,10 +1336,8 @@ user_change_shell_authorized_cb (Daemon                *daemon,
                 argv[4] = user->user_name;
                 argv[5] = NULL;
 
-                error = NULL;
                 if (!spawn_with_login_uid (context, argv, &error)) {
                         throw_error (context, ERROR_FAILED, "running '%s' failed: %s", argv[0], error->message);
-                        g_error_free (error);
                         return;
                 }
 
@@ -1400,9 +1390,9 @@ user_change_icon_file_authorized_cb (Daemon                *daemon,
                                      gpointer               data)
 
 {
-        gchar *filename;
-        GFile *file;
-        GFileInfo *info;
+        g_autofree gchar *filename = NULL;
+        g_autoptr(GFile) file = NULL;
+        g_autoptr(GFileInfo) info = NULL;
         guint32 mode;
         GFileType type;
         guint64 size;
@@ -1411,26 +1401,20 @@ user_change_icon_file_authorized_cb (Daemon                *daemon,
 
         if (filename == NULL ||
             *filename == '\0') {
-                char *dest_path;
-                GFile *dest;
-                GError *error;
+                g_autofree gchar *dest_path = NULL;
+                g_autoptr(GFile) dest = NULL;
+                g_autoptr(GError) error = NULL;
 
-                g_free (filename);
-                filename = NULL;
+                g_clear_pointer (&filename, g_free);
 
                 dest_path = g_build_filename (ICONDIR, user->user_name, NULL);
                 dest = g_file_new_for_path (dest_path);
-                g_free (dest_path);
 
-                error = NULL;
                 if (!g_file_delete (dest, NULL, &error) &&
                     !g_error_matches (error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND)) {
-                        g_object_unref (dest);
                         throw_error (context, ERROR_FAILED, "failed to remove user icon, %s", error->message);
-                        g_error_free (error);
                         return;
                 }
-                g_object_unref (dest);
                 goto icon_saved;
         }
 
@@ -1443,13 +1427,9 @@ user_change_icon_file_authorized_cb (Daemon                *daemon,
         type = g_file_info_get_file_type (info);
         size = g_file_info_get_attribute_uint64 (info, G_FILE_ATTRIBUTE_STANDARD_SIZE);
 
-        g_object_unref (info);
-        g_object_unref (file);
-
         if (type != G_FILE_TYPE_REGULAR) {
                 g_debug ("not a regular file");
                 throw_error (context, ERROR_FAILED, "file '%s' is not a regular file", filename);
-                g_free (filename);
                 return;
         }
 
@@ -1457,41 +1437,34 @@ user_change_icon_file_authorized_cb (Daemon                *daemon,
                 g_debug ("file too large");
                 /* 1MB ought to be enough for everybody */
                 throw_error (context, ERROR_FAILED, "file '%s' is too large to be used as an icon", filename);
-                g_free (filename);
                 return;
         }
 
         if ((mode & S_IROTH) == 0 ||
             (!g_str_has_prefix (filename, DATADIR) &&
              !g_str_has_prefix (filename, ICONDIR))) {
-                gchar *dest_path;
-                GFile *dest;
+                g_autofree gchar *dest_path = NULL;
+                g_autoptr(GFile) dest = NULL;
                 const gchar *argv[3];
                 gint std_out;
-                GError *error;
-                GInputStream *input;
-                GOutputStream *output;
+                g_autoptr(GError) error = NULL;
+                g_autoptr(GInputStream) input = NULL;
+                g_autoptr(GOutputStream) output = NULL;
                 gint uid;
                 gssize bytes;
                 struct passwd *pw;
 
                 if (!get_caller_uid (context, &uid)) {
                         throw_error (context, ERROR_FAILED, "failed to copy file, could not determine caller UID");
-                        g_free (filename);
                         return;
                 }
 
                 dest_path = g_build_filename (ICONDIR, user->user_name, NULL);
                 dest = g_file_new_for_path (dest_path);
 
-                error = NULL;
                 output = G_OUTPUT_STREAM (g_file_replace (dest, NULL, FALSE, 0, NULL, &error));
                 if (!output) {
                         throw_error (context, ERROR_FAILED, "creating file '%s' failed: %s", dest_path, error->message);
-                        g_error_free (error);
-                        g_free (filename);
-                        g_free (dest_path);
-                        g_object_unref (dest);
                         return;
                 }
 
@@ -1501,46 +1474,27 @@ user_change_icon_file_authorized_cb (Daemon                *daemon,
 
                 pw = getpwuid (uid);
 
-                error = NULL;
                 if (!g_spawn_async_with_pipes (NULL, (gchar**)argv, NULL, 0, become_user, pw, NULL, NULL, &std_out, NULL, &error)) {
                         throw_error (context, ERROR_FAILED, "reading file '%s' failed: %s", filename, error->message);
-                        g_error_free (error);
-                        g_free (filename);
-                        g_free (dest_path);
-                        g_object_unref (dest);
                         return;
                 }
 
                 input = g_unix_input_stream_new (std_out, FALSE);
 
-                error = NULL;
                 bytes = g_output_stream_splice (output, input, G_OUTPUT_STREAM_SPLICE_CLOSE_TARGET, NULL, &error);
                 if (bytes < 0 || (gsize)bytes != size) {
                         throw_error (context, ERROR_FAILED, "copying file '%s' to '%s' failed: %s", filename, dest_path, error ? error->message : "unknown reason");
-                        if (error)
-                                g_error_free (error);
-
                         g_file_delete (dest, NULL, NULL);
-
-                        g_free (filename);
-                        g_free (dest_path);
-                        g_object_unref (dest);
-                        g_object_unref (input);
-                        g_object_unref (output);
                         return;
                 }
 
-                g_object_unref (dest);
-                g_object_unref (input);
-                g_object_unref (output);
-
                 g_free (filename);
-                filename = dest_path;
+                filename = g_steal_pointer (&dest_path);
         }
 
 icon_saved:
         g_free (user->icon_file);
-        user->icon_file = filename;
+        user->icon_file = g_steal_pointer (&filename);
 
         save_extra_data (user);
 
@@ -1590,7 +1544,7 @@ user_change_locked_authorized_cb (Daemon                *daemon,
 
 {
         gboolean locked = GPOINTER_TO_INT (data);
-        GError *error;
+        g_autoptr(GError) error = NULL;
         const gchar *argv[5];
 
         if (user->locked != locked) {
@@ -1603,10 +1557,8 @@ user_change_locked_authorized_cb (Daemon                *daemon,
                 argv[3] = user->user_name;
                 argv[4] = NULL;
 
-                error = NULL;
                 if (!spawn_with_login_uid (context, argv, &error)) {
                         throw_error (context, ERROR_FAILED, "running '%s' failed: %s", argv[0], error->message);
-                        g_error_free (error);
                         return;
                 }
 
@@ -1668,11 +1620,12 @@ user_change_account_type_authorized_cb (Daemon                *daemon,
 
 {
         AccountType account_type = GPOINTER_TO_INT (data);
-        GError *error;
+        g_autoptr(GError) error = NULL;
         gid_t *groups;
         gint ngroups;
-        GString *str;
-        gid_t wheel;
+        g_autoptr(GString) str = NULL;
+        g_auto(GStrv) extra_admin_groups = NULL;
+        gid_t admin_gid;
         struct group *grp;
         gint i;
         const gchar *argv[6];
@@ -1684,22 +1637,33 @@ user_change_account_type_authorized_cb (Daemon                *daemon,
 
                 grp = getgrnam (ADMIN_GROUP);
                 if (grp == NULL) {
-                        throw_error (context, ERROR_FAILED, "failed to set account type: wheel group not found");
+                        throw_error (context, ERROR_FAILED, "failed to set account type: " ADMIN_GROUP " group not found");
                         return;
                 }
-                wheel = grp->gr_gid;
+                admin_gid = grp->gr_gid;
 
                 ngroups = get_user_groups (user->user_name, user->gid, &groups);
 
                 str = g_string_new ("");
                 for (i = 0; i < ngroups; i++) {
-                        if (groups[i] == wheel)
+                        if (groups[i] == admin_gid)
                                 continue;
                         g_string_append_printf (str, "%d,", groups[i]);
                 }
                 switch (account_type) {
                 case ACCOUNT_TYPE_ADMINISTRATOR:
-                        g_string_append_printf (str, "%d", wheel);
+                        extra_admin_groups = g_strsplit (EXTRA_ADMIN_GROUPS, ",", 0);
+
+                        for (i = 0; extra_admin_groups[i] != NULL; i++) {
+                                struct group *extra_group;
+                                extra_group = getgrnam (extra_admin_groups[i]);
+                                if (extra_group == NULL || extra_group->gr_gid == admin_gid)
+                                        continue;
+
+                                g_string_append_printf (str, "%d,", extra_group->gr_gid);
+                        }
+
+                        g_string_append_printf (str, "%d", admin_gid);
                         break;
                 case ACCOUNT_TYPE_STANDARD:
                 default:
@@ -1717,12 +1681,8 @@ user_change_account_type_authorized_cb (Daemon                *daemon,
                 argv[4] = user->user_name;
                 argv[5] = NULL;
 
-                g_string_free (str, FALSE);
-
-                error = NULL;
                 if (!spawn_with_login_uid (context, argv, &error)) {
                         throw_error (context, ERROR_FAILED, "running '%s' failed: %s", argv[0], error->message);
-                        g_error_free (error);
                         return;
                 }
 
@@ -1767,7 +1727,7 @@ user_change_password_mode_authorized_cb (Daemon                *daemon,
 
 {
         PasswordMode mode = GPOINTER_TO_INT (data);
-        GError *error;
+        g_autoptr(GError) error = NULL;
         const gchar *argv[6];
 
         if (user->password_mode != mode) {
@@ -1786,10 +1746,8 @@ user_change_password_mode_authorized_cb (Daemon                *daemon,
                         argv[3] = user->user_name;
                         argv[4] = NULL;
 
-                        error = NULL;
                         if (!spawn_with_login_uid (context, argv, &error)) {
                                 throw_error (context, ERROR_FAILED, "running '%s' failed: %s", argv[0], error->message);
-                                g_error_free (error);
                                 return;
                         }
 
@@ -1801,10 +1759,8 @@ user_change_password_mode_authorized_cb (Daemon                *daemon,
                                 argv[4] = user->user_name;
                                 argv[5] = NULL;
 
-                                error = NULL;
                                 if (!spawn_with_login_uid (context, argv, &error)) {
                                         throw_error (context, ERROR_FAILED, "running '%s' failed: %s", argv[0], error->message);
-                                        g_error_free (error);
                                         return;
                                 }
                         }
@@ -1829,10 +1785,8 @@ user_change_password_mode_authorized_cb (Daemon                *daemon,
                         argv[3] = user->user_name;
                         argv[4] = NULL;
 
-                        error = NULL;
                         if (!spawn_with_login_uid (context, argv, &error)) {
                                 throw_error (context, ERROR_FAILED, "running '%s' failed: %s", argv[0], error->message);
-                                g_error_free (error);
                                 return;
                         }
 
@@ -1861,13 +1815,22 @@ user_set_password_mode (AccountsUser          *auser,
 {
         User *user = (User*)auser;
         const gchar *action_id;
+        gint uid;
 
         if (mode < 0 || mode > PASSWORD_MODE_LAST) {
                 throw_error (context, ERROR_FAILED, "unknown password mode: %d", mode);
                 return FALSE;
         }
 
-        action_id = "org.freedesktop.accounts.user-administration";
+        if (!get_caller_uid (context, &uid)) {
+                throw_error (context, ERROR_FAILED, "identifying caller failed");
+                return FALSE;
+        }
+
+        if (user->uid == (uid_t) uid)
+                action_id = "org.freedesktop.accounts.change-own-password";
+        else
+                action_id = "org.freedesktop.accounts.user-administration";
 
         daemon_local_check_auth (user->daemon,
                                  user,
@@ -1889,7 +1852,7 @@ user_change_password_authorized_cb (Daemon                *daemon,
 
 {
         gchar **strings = data;
-        GError *error;
+        g_autoptr(GError) error = NULL;
         const gchar *argv[6];
 
         sys_log (context,
@@ -1905,10 +1868,8 @@ user_change_password_authorized_cb (Daemon                *daemon,
         argv[4] = user->user_name;
         argv[5] = NULL;
 
-        error = NULL;
         if (!spawn_with_login_uid (context, argv, &error)) {
                 throw_error (context, ERROR_FAILED, "running '%s' failed: %s", argv[0], error->message);
-                g_error_free (error);
                 return;
         }
 
@@ -1952,15 +1913,27 @@ user_set_password (AccountsUser          *auser,
 {
         User *user = (User*)auser;
         gchar **data;
+        const gchar *action_id;
+        gint uid;
+
+        if (!get_caller_uid (context, &uid)) {
+                throw_error (context, ERROR_FAILED, "identifying caller failed");
+                return FALSE;
+        }
 
         data = g_new (gchar *, 3);
         data[0] = g_strdup (password);
         data[1] = g_strdup (hint);
         data[2] = NULL;
 
+        if (user->uid == (uid_t) uid)
+                action_id = "org.freedesktop.accounts.change-own-password";
+        else
+                action_id = "org.freedesktop.accounts.user-administration";
+
         daemon_local_check_auth (user->daemon,
                                  user,
-                                 "org.freedesktop.accounts.user-administration",
+                                 action_id,
                                  TRUE,
                                  user_change_password_authorized_cb,
                                  context,
@@ -2036,7 +2009,7 @@ user_change_automatic_login_authorized_cb (Daemon                *daemon,
                                            gpointer               data)
 {
         gboolean enabled = GPOINTER_TO_INT (data);
-        GError *error = NULL;
+        g_autoptr(GError) error = NULL;
 
         sys_log (context,
                  "%s automatic login for user '%s' (%d)",
@@ -2049,7 +2022,6 @@ user_change_automatic_login_authorized_cb (Daemon                *daemon,
 
         if (!daemon_local_set_automatic_login (daemon, user, enabled, &error)) {
                 throw_error (context, ERROR_FAILED, "failed to change automatic login: %s", error->message);
-                g_error_free (error);
                 return;
         }
 
@@ -2213,8 +2185,7 @@ user_finalize (GObject *object)
         g_free (user->location);
         g_free (user->password_hint);
 
-	if (user->login_history)
-		g_variant_unref (user->login_history);
+	g_clear_pointer (&user->login_history, g_variant_unref);
 
         if (G_OBJECT_CLASS (user_parent_class)->finalize)
                 (*G_OBJECT_CLASS (user_parent_class)->finalize) (object);
